# 
# $Header: usm/src/cmds/acfsroot/acfsroot.pl /st_usm_11.2.0/1 2010/07/01 11:53:11 certan Exp $
#
# acfsroot.pl
# 
# Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved. 
#
#    NAME
#      acfsroot.pl
#
#    DESCRIPTION
#      install/uninstall USM components from the distribution files
#      See section 4.2.1 of Design Note Supporting USM Installation
#
#    NOTES
#      acfsroot install -l -k -h -s
#          install USM kernel drivers and commands.
#      acfsroot uninstall -h -s
#          uninstall USM kernel drivers and commands.
#      acfsroot version_check -l -k -h -s
#          check if USM components are available for installation.
#
#    INSTALL ACTIONS
#      - verify that the user has root privs.
#      - checks that the proper install files exists
#      - unloads currently loaded drivers (if any).
#      - removes currently installed USM install files (if any).
#      - installs from the selected distribution files
#      - Loads the drivers to make sure that they load properly.
#      - Performs any required OSD actions, if needed.
#
#      User output should be done in this file when possible to ensure
#      cross platform similarity in command look and feel. However, it is 
#      understood that some OSD actions are specific to one platform only,
#      in which case it makes sense to do the output there.
#
#    MODIFIED   (MM/DD/YY)
#    certan      06/28/10 - Pass $install_files_loc to uninstall() and
#                           osds_usm_uninstall() as distribution files may be
#                           needed during uninstall process
#    jinjche     06/02/10 - Add reregister and unregister subcommands for
#                           managing CRS ACFS resources
#    agraves     04/10/10 - Change message 9316 to include the media location.
#    averhuls    03/17/10 - Return USM_NOT_SUPPORTED if the platform does not
#                           support USM.
#    anjiwaji    02/10/10 - Change install and uninstall to check if any
#                           drivers are running.
#    certan      11/25/09 - During the install process, pass the location
#                           of install files to lib_unload_usm_drivers so
#                           that new utilities can be used if the old
#                           utilities cannot be found
#    agraves     04/30/09 - Make messages be consistent with Bill Manry's
#                           review.
#    averhuls    03/17/09 - Correctly detect NT absolute paths in
#                           get_oracle_home().
#    averhuls    03/16/09 - Fix wrapper scripts if USM is not supported on
#                           a machine.
#    averhuls    02/19/09 - Remove lib_osds references - everything goes thru
#                           lib.
#    averhuls    01/30/09 - rename usm_ to acfs.
#    averhuls    01/21/09 - For version_check, list where it is searching for
#                           media.
#    averhuls    12/23/08 - Use message catalog.
#    averhuls    11/06/08 - Fix Windows shiphome problem where the
#                           "lib" directory might be "LIB".
#    averhuls    11/03/08 - Use library functions from usm_lib.pm.
#    averhuls    10/20/08 - Remove "USM" from print statements.
#    averhuls    10/14/08 - Search for usm_driver_state.pl not usm_driver_state
#                           because that is common to all platforms.
#    averhuls    08/26/08 - convert constants implemented as variables to real
#                           constants
#    averhuls    07/25/08 - Fix uninitialized error if executed on Redhat4.
#                           -s only prints error messages (bug 7279974).
#    averhuls    07/02/08 - remove the -u option.
#    averhuls    04/11/08 - fix minor problems found during beta.
#    averhuls    03/13/08 - Improve printing. change usm_log_error() calls
#                           to usm_print(). Use usm_driver_state rather
#                           than an internal OSD function to determine
#                           if USM drivers are loaded.
#    averhuls    02/14/08 - minor cleanup.
#    averhuls    12/18/07 - Enhancement. if no install media found, list where
#                           it was liiking for them
#    averhuls    11/12/07 - move root validation to osd code
#    averhuls    10/16/07 - Look for media in the default location unless -l is
#                           specified.
#    averhuls    10/08/07 - Add switch to tell if a suitable RPM is available.
#    averhuls    06/29/07 - Creation
# 

use strict;
use Getopt::Std;
use Cwd;
use File::Basename;
use Switch;
use English;
use acfslib;
use osds_acfsroot;

sub main
{
  my ($sub_command);         # start or stop
  my ($return_code);         # as the name implies 
  # user options
  my (%opt);                 # user options hash - used by getopts()
  my ($install_kernel_vers); # -k : install kernel version (other than current)
  my ($install_files_loc);   # -l : location(s) of the distribution files 
                             # -s : silent operation - no console output 
                             # -h : help

  # user flags. See description above or usage message output
  my (%flags) = ( install       => 'k:l:hs',
                  uninstall     => 'hs',
                  reregister    => 'hs',
                  unregister    => 'hs',
                  version_check => 'k:l:hs',
                );

  # supplied by the front end driver and 'guaranteed' to be there.
  # command is what the user actually typed in (sans directories).
  $COMMAND = shift(@ARGV);

  # supplied by user
  $sub_command = shift(@ARGV);

  # determine $ORACLE_HOME for this system
  get_oracle_home();

  if (!lib_usm_supported())
  {
    # OSD specific message generated in lib_usm_supported().

    # Resolve ORACLE_HOME in the "wrapper scripts".
    osds_fix_wrapper_scripts();
    exit USM_NOT_SUPPORTED;
  }

  # sub commands "install", "uninstall", "reregister", "unregister", or "version_check"
  # must be supplied by the user.
  if (defined($sub_command))
  {
    if (!(($sub_command eq 'install') ||
          ($sub_command eq 'uninstall') ||
          ($sub_command eq 'reregister') ||
          ($sub_command eq 'unregister') ||
          ($sub_command eq 'version_check')))
    {
      # illegal sub-command
      usage("invalid", 0);
      exit USM_FAIL;
    }
  }
  else
  {
    # no sub-command
    usage("invalid", 0);
    exit USM_FAIL;
  }

  # parse user options
  %opt=();
  getopts($flags{$sub_command}, \%opt) or usage($sub_command, 1);
  if ($opt{'k'})
  {
    $install_kernel_vers = $opt{'k'};
  }
  if ($opt{'l'})
  {
    $install_files_loc = $opt{'l'};
  }
  if ($opt{'h'})
  {
    # print help information
    usage($sub_command, 0);
    return USM_SUCCESS;
  }
  if ($opt{'s'})
  {
    # do not generate console output - default is not silent.
    $SILENT = 1;
  }

  ##### command parsing complete #####

  # perform required OSD initialization, if any.
  osds_initialize($install_kernel_vers);

  # use the default location for media unless the user specified otherwise
  if (!defined($install_files_loc))
  {
    $install_files_loc = $USM_DFLT_MEDIA_LOC; 
  }

  # version availability checks don't require privileged access
  if ($sub_command eq 'version_check')
  {
    # check the availability of USM components
    $return_code = version_check($install_kernel_vers, $install_files_loc);
    exit $return_code;
  }

  # verify root access
  if (!lib_am_root())
  {
    lib_error_print(9130, "Root access required");
    exit USM_FAIL;
  }

  if ($sub_command eq 'install')
  {
    # install the USM components
    $return_code = install($install_kernel_vers, $install_files_loc);
  }
  else
  {
    if ($sub_command eq 'uninstall')
    {
      # uninstall the USM components
      # pass $install_files_loc to uninstall() as distribution files may be
      # needed during uninstall process
      $return_code = uninstall($install_files_loc);
    }
    else
    {
      if ($sub_command eq 'reregister')
      {
        # reregister the ACFS resources
        $return_code = reregister();
      }
      else
      {
        if ($sub_command eq 'unregister')
        {
          # unregister the ACFS resources
          $return_code = unregister();
        }
      }
    }
  }

  exit $return_code;
} # end main

sub install
{
  my ($install_kernel_vers, $install_files_loc) = @_;
  my ($no_load) = 0;
  my ($return_code);
  my ($kernel_version) = osds_get_kernel_version();

  if (defined($install_kernel_vers))
  {
    # We're installing USM for another kernel version - do not attempt to
    # load the drivers. The presumed scenario is that the user wants to
    # install USM for an about to be upgraded kernel. This way, USM can
    # be up and running upon reboot. Dunno if anyone will ever use this.
    $kernel_version = $install_kernel_vers;
    $no_load = 1;
  }  
  
  # First, find the distribution files from which to install
  # No point in going on if they can't be found.
  $return_code = osds_search_for_distribution_files($install_files_loc);
  if ($return_code == USM_SUCCESS)
  {
    lib_inform_print(9300, "ADVM/ACFS distribution files found.");
  }
  else
  {
    lib_error_print(9301, "ADVM/ACFS installation can not proceed:");
    if (defined($install_files_loc))
    {
      lib_error_print(9317,
                     "No ADVM/ACFS distribution media detected at " .
                     "location: '%s'", $install_files_loc);
    }
    else
    {
      lib_error_print(9303,
         "No installation files found for OS kernel version %s.", $kernel_version);
    }
    return USM_FAIL;

  }

  # Can't continue if the currently loaded drivers can't be unloaded
  if (lib_unload_usm_drivers($install_files_loc) == USM_FAIL)
  {
    lib_error_print(9304,
          "Installation can not proceed: Failed to unload ADVM/ACFS drivers.");
    return USM_FAIL;
  }

  # Search for a previous installation and remove it if found.
  if (lib_check_any_driver_installed())
  {
    # Pass $install_files_loc to uninstall() as distribution files may be
    # needed during uninstall process
    if (uninstall($install_files_loc) == USM_FAIL)
    {
      lib_error_print(9305, "ADVM/ACFS installation can not proceed:");
      lib_error_print(9306, "Failed to uninstall previous installation.");
      return USM_FAIL;
    }
  }

  # We have distribution files and no USM components are currently 
  # installed or loaded - we can proceed with the installation.
  lib_inform_print(9307, "Installing requested ADVM/ACFS software.");

  # osds_search_for_distribution files() has set which files need
  # to be installed.
  $return_code = osds_install_from_distribution_files();

  if ($return_code == USM_SUCCESS)
  {
    if ($no_load == 0)
    {
      lib_inform_print (9308, "Loading installed ADVM/ACFS drivers.");
    }

    # ensure that all utilities and drivers are where they are expected to be
    $return_code = osds_load_and_verify_usm_state($no_load);
    if ($return_code == USM_SUCCESS)
    {
      lib_inform_print(9309, "ADVM/ACFS installation correctness verified.");
    }
    else
    {
      lib_error_print(9310, "ADVM/ACFS installation failed.");
      lib_error_print(9311,
                   "not all components were detected after the installation.");
    }
  }
  else
  {
    lib_error_print(9310, "ADVM/ACFS installation failed.");
  }

  return $return_code;
} # end install

# Re-register ACFS drivers and registry resources
sub reregister
{
  my $ret = USM_SUCCESS;

  my $ret1 = add_usm_drivers_resource();
  if ($ret1 != USM_SUCCESS)
  {
    $ret = USM_FAIL;
  }

  $ret1 = start_usm_drivers_resource();
  if ($ret1 != USM_SUCCESS)
  {
    $ret = USM_FAIL;
  }

  $ret1 = add_acfs_registry();
  if ($ret1 != USM_SUCCESS)
  {
    $ret = USM_FAIL;
  }

  $ret1 = start_acfs_registry();
  if ($ret1 != USM_SUCCESS)
  {
    $ret = USM_FAIL;
  }

  return $ret;
} 

# Unregister ACFS drivers and registry resources
sub unregister
{
  my $ret = USM_SUCCESS;

  my $ret1 = stop_acfs_registry();
  if ($ret1 != USM_SUCCESS)
  {
    $ret = USM_FAIL;
  }

  $ret1 = delete_acfs_registry();
  if ($ret1 != USM_SUCCESS)
  {
    $ret = USM_FAIL;
  }

  $ret1 = delete_usm_drivers_resource();
  if ($ret1 != USM_SUCCESS)
  {
    $ret = USM_FAIL;
  }

  return $ret;
}

sub uninstall
{
  my ($install_files_loc) = @_;
  my ($return_code);

  # Search for a previous installation
  if (lib_check_any_driver_installed())
  {
    lib_inform_print(9312, "Existing ADVM/ACFS installation detected.");
  }
  else
  {
    lib_error_print(9313, "No ADVM/ACFS installation detected.");
    return USM_SUCCESS;
  }

  # Can't continue if the currently loaded drivers can't be unloaded
  if (lib_unload_usm_drivers() == USM_FAIL)
  {
    return USM_FAIL;
  }

  lib_inform_print(9314, "Removing previous ADVM/ACFS installation.");

  # Pass $install_files_loc to osds_usm_uninstall() as distribution files may
  # be needed during uninstall process
  $return_code = osds_usm_uninstall($install_files_loc);
  if ($return_code == USM_SUCCESS)
  {
    lib_inform_print(9315,
                       "Previous ADVM/ACFS components successfully removed."); 
  }

  return $return_code;
} # end uninstall

sub version_check
{
  my ($install_kernel_vers, $install_files_loc) = @_;
  my ($return_code);
  my ($kernel_version) = osds_get_kernel_version();

  if (defined($install_kernel_vers))
  {
    # We're checking ADVM/ACFS for another kernel version
    $kernel_version = $install_kernel_vers;
  }  

  $return_code = osds_search_for_distribution_files(
                      $install_files_loc, $kernel_version);

  if ($return_code == 0)
  {
    lib_inform_print(9316, 
          "Valid ADVM/ACFS distribution media detected at: '%s'",
          $MEDIA_FOUND);
  }
  else
  {
    if (defined($MEDIA_FOUND))
    {
      lib_error_print(9317, "No ADVM/ACFS distribution media detected at " .
                            "location: '%s'", $MEDIA_FOUND);
    }
    else
    {
      lib_error_print(9317, "No ADVM/ACFS distribution media detected at " .
                            "location: '%s'", "unknown");
    }
  }

  return $return_code;
} # end check

################################################
# The following are static functions.
################################################

# On a production system, acfsroot.pl lives in $ORACLE_HOME/lib. We can't use 
# $ORACLE_HOME directly becasue we can't be guaranteed of the environment this
# command is called in. So, in a shiphome, we use the command location to
# determine the proper $ORACLE_HOME.
#
sub get_oracle_home
{
  my ($dir) = dirname($0);          # $0 is built in acfstoolsdriver.{sh,bat}

  if (defined($ENV{SRCHOME}))
  {
    # We're in a development environment, we'll use that $ORACLE_HOME.
    $ORACLE_HOME = $ENV{ORACLE_HOME};
    return;
  }

  # We're in a production environment.

  # This file lives in $ORACLE_HOME/lib - drop the trailing /lib
  $dir =~ s/\/lib$//;

  $ORACLE_HOME = $dir;
  $ENV{ORACLE_HOME} = $ORACLE_HOME;
}

sub usage
{
  my ($sub_command, $abort) = @_;

  lib_error_print(9134, "usage:");

  if (($sub_command eq "install") || ($sub_command eq "version_check"))
  {
    lib_error_print(9331,
                 " %s [-l <location>] [-k <version>] [-h] [-s]", $sub_command);
    lib_error_print(9332, "    -l : location(s) of the distribution files");
    lib_error_print(9333, "         (if not the default location)");
    lib_error_print(9334,
            "         (comma separated if more than one location specified)");
    lib_error_print(9335, "    -k : install kernel version number");
    lib_error_print(9336, "         (if not the current kernel version)");
    lib_error_print(9132, "    -h print help/usage information");
    lib_error_print(9131,
               "    -s silent - no informational messages - errors only");
  }
  elsif ($sub_command eq "uninstall")
  {
    lib_error_print(9338, "    uninstall [-h] [-s]");
    lib_error_print(9132, "    -h print help/usage information");
    lib_error_print(9131,
               "    -s silent - no informational messages - errors only");
  }
  else
  {
    lib_error_print(9337,
                       " <install | uninstall | reregister | unregister | version_check > [arguments]");
  }

  if ($abort)
  {
    exit USM_FAIL;
  }
} # end usage

main();

